package com.ruoyi.web.controller.system;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.ruoyi.common.annotation.Log;
import com.ruoyi.common.constant.CacheConstants;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.core.domain.entity.SysUser;
import com.ruoyi.common.core.domain.model.LoginUser;
import com.ruoyi.common.core.redis.RedisCache;
import com.ruoyi.common.enums.BusinessType;
import com.ruoyi.common.exception.user.CaptchaException;
import com.ruoyi.common.exception.user.CaptchaExpireException;
import com.ruoyi.common.exception.user.CaptchaNullException;
import com.ruoyi.common.exception.user.ProjectException;
import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.common.utils.SecurityUtils;
import com.ruoyi.common.utils.StringUtils;
import com.ruoyi.common.utils.file.OssUtils;
import com.ruoyi.framework.web.service.TokenService;
import com.ruoyi.generator.domain.Community.CommunityUserBlock;
import com.ruoyi.generator.domain.Community.CommunityUserInfo;
import com.ruoyi.generator.mapper.community.CommunityUserInfoMapper;
import com.ruoyi.generator.service.ICommunityUserBlockService;
import com.ruoyi.system.mapper.SysUserMapper;
import com.ruoyi.system.service.ISysUserService;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import java.util.Objects;

/**
 * 个人信息 业务处理
 *
 * @author ruoyi
 */
@RestController
@RequestMapping("/system/user/profile")
public class SysProfileController extends BaseController {
    @Autowired
    private ISysUserService userService;

    @Autowired
    private TokenService tokenService;

    @Autowired
    private CommunityUserInfoMapper communityUserInfoMapper;

    @Autowired
    private ICommunityUserBlockService communityUserBlockService;

    private static final String fileUrl = "http://file.iciyuanshidai.com";

    @Autowired
    private SysUserMapper userMapper;

    @Autowired
    private RedisCache redisCache;

    /**
     * 个人信息
     */
    @GetMapping
    public AjaxResult profile() {
        LoginUser loginUser = getLoginUser();
        SysUser user = loginUser.getUser();
        AjaxResult ajax = AjaxResult.success(user);
        ajax.put("roleGroup", userService.selectUserRoleGroup(loginUser.getUsername()));
        ajax.put("postGroup", userService.selectUserPostGroup(loginUser.getUsername()));
        return ajax;
    }

    /**
     * 修改用户
     */
    @Log(title = "个人信息", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult updateProfile(@RequestBody SysUser user) {
        LoginUser loginUser = getLoginUser();
        SysUser currentUser = loginUser.getUser();
        currentUser.setNickName(user.getNickName());
        currentUser.setEmail(user.getEmail());
        currentUser.setPhonenumber(user.getPhonenumber());
        currentUser.setSex(user.getSex());
        currentUser.setProfile(user.getProfile());
        currentUser.setAvatar(user.getAvatar());
        if (StringUtils.isNotEmpty(user.getPhonenumber()) && !userService.checkPhoneUnique(currentUser)) {
            return error("修改用户'" + loginUser.getUsername() + "'失败，手机号码已存在");
        }
        if (StringUtils.isNotEmpty(user.getEmail()) && !userService.checkEmailUnique(currentUser)) {
            return error("修改用户'" + loginUser.getUsername() + "'失败，邮箱账号已存在");
        }

        //查询用户是否还有修改性别的次数
       /* SysUser sysUsers = userMapper.selectOne(
                new QueryWrapper<SysUser>()
                        .select("sex", "is_sex") // 选择 sex 和 is_sex 字段
                        .eq("user_id", currentUser.getUserId())
        );

        if (sysUsers.getIsSex() && !currentUser.getSex().equals(sysUsers.getSex())) {
            return error("不允许修改用户'" + loginUser.getUsername() + "'性别");
        } else if (currentUser.getSex().equals(sysUsers.getSex())) {
            currentUser.setSex(null);
        }*/

        if (userService.updateUserProfile(currentUser) > 0) {
            CommunityUserInfo communityUserInfo = communityUserInfoMapper.selectOne(new QueryWrapper<CommunityUserInfo>().eq("user_id", currentUser.getUserId()));
            communityUserInfo.setProfile(currentUser.getProfile());
            communityUserInfo.setBirthday(user.getBirthday());
            communityUserInfo.setTags(user.getTags());
            communityUserInfo.setUpdateBy(currentUser.getUserId());
            communityUserInfo.setUpdateTime(DateUtils.parseDate(DateUtils.getTime()));
            communityUserInfo.setBackImage(user.getBackImage());
            //更新前端用户的个人简介信息
            communityUserInfoMapper.updateById(communityUserInfo);

            // 更新缓存用户信息
            tokenService.setLoginUser(loginUser);
            return success();
        }
        return error("修改个人信息异常，请联系管理员");
    }

    /**
     * 重置密码
     */
    @Log(title = "个人信息", businessType = BusinessType.UPDATE)
    @PutMapping("/updatePwd")
    public AjaxResult updatePwd(String oldPassword, String newPassword) {
        LoginUser loginUser = getLoginUser();
        String userName = loginUser.getUsername();
        String password = loginUser.getPassword();
        if (!SecurityUtils.matchesPassword(oldPassword, password)) {
            return error("修改密码失败，旧密码错误");
        }
        if (SecurityUtils.matchesPassword(newPassword, password)) {
            return error("新密码不能与旧密码相同");
        }
        newPassword = SecurityUtils.encryptPassword(newPassword);
        if (userService.resetUserPwd(userName, newPassword) > 0) {
            // 更新缓存用户密码
            loginUser.getUser().setPassword(newPassword);
            tokenService.setLoginUser(loginUser);
            return success();
        }
        return error("修改密码异常，请联系管理员");
    }

    /**
     * 重置密码(验证码)
     */
    @Log(title = "个人信息", businessType = BusinessType.UPDATE)
    @PutMapping("/updatePwdBySmsCode")
    public AjaxResult updatePwdBySmsCode(String username, String smsCode, String newPassword) {
        if (StringUtils.isEmpty(newPassword) || StringUtils.isEmpty(smsCode) || StringUtils.isEmpty(username)) {
            return error("参数异常!");
        }

        validateSmsCaptcha(username, smsCode);
        newPassword = SecurityUtils.encryptPassword(newPassword);
        int result = userService.resetUserPwd(username, newPassword);
        if (result > 0) {
            return success();
        }
        return error("修改密码异常，请联系管理员");
    }

    /**
     * 头像上传
     */
    @Log(title = "用户头像", businessType = BusinessType.UPDATE)
    @PostMapping("/avatar")
    public AjaxResult avatar(@RequestParam("avatarfile") MultipartFile file) throws Exception {
        if (!file.isEmpty()) {
            LoginUser loginUser = getLoginUser();
//            String avatar = FileUploadUtils.upload(RuoYiConfig.getAvatarPath(), file, MimeTypeUtils.IMAGE_EXTENSION);
            String avatar = OssUtils.uploadMultipartFile(file);
            avatar = avatar.replace("https://cysd.oss-cn-shanghai.aliyuncs.com", fileUrl);
            if (userService.updateUserAvatar(loginUser.getUsername(), avatar)) {
                AjaxResult ajax = AjaxResult.success();
                ajax.put("imgUrl", avatar);
                // 更新缓存用户头像
                loginUser.getUser().setAvatar(avatar);
                tokenService.setLoginUser(loginUser);
                return ajax;
            }
        }
        return error("上传图片异常，请联系管理员");
    }

    /**
     * 校验验证码
     *
     * @param username 用户名
     * @param code     验证码
     * @return 结果
     */
    public void validateSmsCaptcha(String username, String code) {
        if (StringUtils.isEmpty(code)) {
            throw new CaptchaNullException();
        }
        String verifyKey = CacheConstants.SMS_UPDATE_PASSWORD_CODE_KEY + StringUtils.nvl(username, "");
        try {
            String captcha = redisCache.getCacheObject(verifyKey).toString();
            if (captcha == null) {
                throw new CaptchaExpireException();
            }
            if (!code.equalsIgnoreCase(captcha)) {
                throw new CaptchaException();
            }
            redisCache.deleteObject(verifyKey);
        } catch (NullPointerException e) {
            throw new CaptchaExpireException();
        }
    }


    /**
     * 拉黑用户
     */
    @ApiOperation("拉黑用户")
    @PostMapping("/blockUser")
    @Transactional
    //@Anonymous
    public AjaxResult blockTag(@RequestBody CommunityUserBlock communityUserBlock) {
        if (Objects.isNull(communityUserBlock.getUserId()) || Objects.isNull(communityUserBlock.getPeerId())) {
            return AjaxResult.error("参数异常!");
        }
        CommunityUserBlock communityTagBlock = null;
        try {
            communityTagBlock = communityUserBlockService.blockUser(communityUserBlock);
        } catch (Exception e) {
            throw new ProjectException();
        }

        return AjaxResult.success(communityTagBlock);
    }




}
