package com.ruoyi.generator.controller;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.core.domain.entity.SysUser;
import com.ruoyi.common.core.text.Convert;
import com.ruoyi.common.exception.user.ProjectException;
import com.ruoyi.common.utils.*;
import com.ruoyi.common.utils.ip.AddressUtils;
import com.ruoyi.generator.domain.Community.CommunityArticle;
import com.ruoyi.generator.domain.Community.CommunityArticleComment;
import com.ruoyi.generator.domain.Community.CommunityCommentLike;
import com.ruoyi.generator.domain.Community.CommunityCommentReply;
import com.ruoyi.generator.mapper.community.CommunityArticleCommentMapper;
import com.ruoyi.generator.mapper.community.CommunityArticleMapper;
import com.ruoyi.generator.mapper.community.CommunityCommentLikeMapper;
import com.ruoyi.generator.mapper.community.CommunityCommentReplyMapper;
import com.ruoyi.generator.service.ICommunityArticleCommentService;
import com.ruoyi.generator.service.ICommunityArticleService;
import com.ruoyi.generator.service.ICommunityCommentReplyService;
import com.ruoyi.generator.vo.CommunityArticleCommentVo;
import com.ruoyi.generator.vo.CommunityCommentRaffleVo;
import com.ruoyi.generator.vo.CommunityCommentReplyVo;
import com.ruoyi.generator.vo.CommunitySearchCommentVo;
import com.ruoyi.system.mapper.SysUserMapper;
import com.ruoyi.system.service.impl.SysUserServiceImpl;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.*;

import java.text.ParseException;
import java.util.ArrayList;
import java.util.List;
import java.util.Objects;
import java.util.concurrent.ExecutionException;
import java.util.stream.Collectors;

/**
 * 评论接口
 *
 * @author fangzhen
 */
@Api(tags = "评论接口")
@RestController
@RequestMapping("/community/comment")
public class CommunityCommentController extends BaseController {

    @Autowired
    private ICommunityCommentReplyService communityCommentReplyService;

    @Autowired
    private ICommunityArticleCommentService communityArticleCommentService;

    @Autowired
    private SysUserMapper sysUserMapper;

    @Autowired
    private CommunityCommentLikeMapper communityCommentLikeMapper;

    @Autowired
    private SysUserServiceImpl sysUserServiceImpl;

    @Autowired
    private SysUserServiceImpl sysUserService;

    @Autowired
    private ICommunityArticleService communityArticleService;

    @Autowired
    private CommunityArticleMapper communityArticleMapper;

    @Autowired
    private CommunityArticleCommentMapper communityArticleCommentMapper;

    @Autowired
    private CommunityCommentReplyMapper communityCommentReplyMapper;

    /**
     * 获取文章评论
     */
    @ApiOperation("获取文章评论")
    @GetMapping()
    @Transactional
    //@Anonymous
    public AjaxResult comment(String articleId, int currentPage, int limit) {
        if (Objects.isNull(articleId)) {
            return AjaxResult.error("参数异常!");
        }
        List<CommunityArticleCommentVo> commentVos = new ArrayList<>();
        try {
            Page<CommunityArticleComment> page = new Page<>(currentPage, limit);
            List<CommunityArticleComment> records = communityArticleCommentService.page(page, new QueryWrapper<CommunityArticleComment>()
                    .eq("article_id", articleId)
                    .and((wrapper) -> {
                        wrapper.ne("is_delete", true).or().isNull("is_delete");
                    }).orderByDesc("create_time")).getRecords();

            CommunityArticleCommentVo commentVo = null;

            for (CommunityArticleComment record : records) {
                if (record.getAddress() == null) {
                    record.setAddress("未知");
                }

                commentVo = new CommunityArticleCommentVo();
                BeanUtils.copyProperties(record, commentVo);
                commentVos.add(commentVo);
            }

            Long userId = SecurityUtils.getUserId();
            List<Long> commentIds = new ArrayList<>();
            for (CommunityArticleCommentVo vo : commentVos) {
                Long commentUserId = vo.getUserId();
                //获取评论的用户信息
                SysUser sysUser = sysUserMapper.selectUserById(commentUserId);
                vo.setNickName(sysUser.getNickName());
                vo.setAvatar(sysUser.getAvatar());
                //当前登录用户是否已点赞
                List<CommunityCommentLike> commentLikes = communityCommentLikeMapper.selectList(new QueryWrapper<CommunityCommentLike>()
                        .eq("comment_id", vo.getId())
                        .eq("user_id", userId));
                vo.setCommentLike(!commentLikes.isEmpty());
                //该评论的点赞数量
                vo.setCommentLikeCount(communityCommentLikeMapper.selectList(new QueryWrapper<CommunityCommentLike>()
                                .eq("comment_id", vo.getId()))
                        .size());
                //设置首条评论
                List<CommunityCommentReply> replies = communityCommentReplyService.list(new QueryWrapper<CommunityCommentReply>()
                        .eq("comment_id", vo.getId())
                        .and((wrapper) -> {
                            wrapper.ne("is_delete", true).or().isNull("is_delete");
                        }).orderByAsc("create_time"));

                //如果评价用户id与当前登录用户id相符,则标记为已读
                if (vo.getUserId().equals(userId) && !vo.isRead()) {
                    commentIds.add(vo.getId());
                }

                CommunityCommentReplyVo replyVo = null;
                List<CommunityCommentReplyVo> replyVos = new ArrayList<>();
                for (CommunityCommentReply reply : replies) {
                    replyVo = new CommunityCommentReplyVo();
                    BeanUtils.copyProperties(reply, replyVo);
                    replyVo.setNickName(sysUserService.selectUserById(replyVo.getUserId()).getNickName());
                    replyVo.setReply(true);
                    replyVo.setAvatar(sysUserService.selectUserById(replyVo.getUserId()).getAvatar());
                    replyVos.add(replyVo);
                    //设置点赞数量和当前登录用户是否已点赞
                    List<CommunityCommentLike> commentLikeList = communityCommentLikeMapper.selectList(new QueryWrapper<CommunityCommentLike>()
                            .eq("reply_id", replyVo.getId()));
                    replyVo.setLikeCount(commentLikeList.size());
                    replyVo.setLike(commentLikeList.stream().anyMatch(item -> SecurityUtils.getUserId().equals(item.getUserId())));
                    break;
                }
                vo.setFirstReply(replyVos);
                //设置还剩多少评论
                vo.setReplyCount(replies.isEmpty() ? 0 : replies.size() - 1);
                vo.setReply(false);
            }

            //更新为已读
            if (!commentIds.isEmpty()) {
                communityArticleCommentService.update(new UpdateWrapper<CommunityArticleComment>().set("is_read", true).in("id", commentIds));
            }
        } catch (Exception e) {
            System.out.println(e.getMessage());
            throw new ProjectException();
        }

        return AjaxResult.success(commentVos);
    }

    /**
     * 获取评论回复
     */
    @ApiOperation("获取评论回复")
    @GetMapping("/reply")
    @Transactional
    //@Anonymous
    public AjaxResult reply(String commentId, int currentPage, int limit) {
        if (Objects.isNull(commentId)) {
            return AjaxResult.error("参数异常!");
        }
        List<CommunityCommentReplyVo> replyVos = new ArrayList<>();
        try {
            Long userId = SecurityUtils.getUserId();
            Page<CommunityCommentReply> page = new Page<>(currentPage, limit);
            List<CommunityCommentReply> replies = communityCommentReplyService.page(page, new QueryWrapper<CommunityCommentReply>()
                    .eq("comment_id", commentId)
                    .and((wrapper) -> {
                        wrapper.ne("is_delete", true)
                                .or()
                                .isNull("is_delete");
                    }).orderByAsc("create_time")).getRecords();

            CommunityCommentReplyVo replyVo = null;
            List<Long> replyIds = new ArrayList<>();
            for (final CommunityCommentReply reply : replies) {
                replyVo = new CommunityCommentReplyVo();
                if (reply.getAddress() == null) {
                    reply.setAddress("未知");
                }

                //标记当前登录的用户被回复的信息
                if (reply.getUserId().equals(userId)) {
                    List<CommunityCommentReply> communityCommentReplies = replies.stream()
                            .filter(item -> item.getParentReplyId().equals(reply.getId()))
                            .collect(Collectors.toList());
                    for (CommunityCommentReply communityCommentReply : communityCommentReplies) {
                        replyIds.add(communityCommentReply.getId());
                    }
                }

                SysUser sysUser = sysUserService.selectUserById(reply.getUserId());
                final Long parentReplyId = reply.getParentReplyId();
                BeanUtils.copyProperties(reply, replyVo);
                if (Objects.nonNull(parentReplyId) && !commentId.equalsIgnoreCase(Long.toString(parentReplyId))) {
                    List<CommunityCommentReply> communityCommentReplies = replies.stream()
                            .filter(item -> item.getId().equals(parentReplyId))
                            .collect(Collectors.toList());
                    CommunityCommentReply communityCommentReply = communityCommentReplies.get(0);
                    SysUser sysUser1 = sysUserService.selectUserById(communityCommentReply.getUserId());
                    replyVo.setParentNickName(sysUser1.getNickName());
                    replyVo.setParentUserId(sysUser1.getUserId());
                    replyVo.setParentAvatar(sysUser1.getAvatar());
                }
                replyVo.setNickName(sysUser.getNickName());
                replyVo.setAvatar(sysUser.getAvatar());
                replyVo.setReply(true);
                //设置点赞数量和当前登录用户是否已点赞
                List<CommunityCommentLike> commentLikes = communityCommentLikeMapper.selectList(new QueryWrapper<CommunityCommentLike>()
                        .eq("reply_id", replyVo.getId()));
                replyVo.setLikeCount(commentLikes.size());
                replyVo.setLike(commentLikes.stream().anyMatch(item -> SecurityUtils.getUserId().equals(item.getUserId())));
                replyVos.add(replyVo);
            }

            //更新回复的id为已读
            if (!replyIds.isEmpty()) {
                communityCommentReplyService.update(new UpdateWrapper<CommunityCommentReply>()
                        .set("is_read", true)
                        .in("id", replyIds));
            }
        } catch (Exception e) {
            System.out.println(e.getMessage());
            throw new ProjectException();
        }
        return AjaxResult.success(replyVos);
    }


    /**
     * 对文章评论进行回复
     */
    @ApiOperation("对文章评论进行回复")
    @PostMapping("/reply")
    @Transactional
    //@Anonymous
    public AjaxResult reply(@RequestBody CommunityCommentReply communityCommentReply) throws ParseException {
        if (Objects.isNull(communityCommentReply.getCommentId()) || Objects.isNull(communityCommentReply.getContent())) {
            return AjaxResult.error("参数异常!");
        }

        Long userId = SecurityUtils.getUserId();
        try {
            //过滤敏感词
            boolean isSensitive = SensitiveWordUtil.containsSensitiveWord(communityCommentReply.getContent());
            if (isSensitive) {
                return AjaxResult.error(MessageUtils.message("article.user.comment"));
            }

            Boolean aBoolean = communityArticleService.checkCommentPermission(userId, communityCommentReply.getCommentId(), null);
            if (!aBoolean) {  // 当 aBoolean 为 false 时，进入这个条件
                return AjaxResult.success("您没有权限评论此文章，因为作者只允许关注的人评论!");
            }

            communityCommentReply.setUserId(userId);
            communityCommentReply.setDelete(false);
            communityCommentReply.setCreateBy(userId);
            communityCommentReply.setCreateTime(DateUtils.parseDate(DateUtils.getTime()));
            String address = null;
            try {
                address = AddressUtils.getAddress(null);
            } catch (ExecutionException e) {
                throw new RuntimeException(e);
            } catch (InterruptedException e) {
                throw new RuntimeException(e);
            }
            communityCommentReply.setAddress(address);
            communityCommentReplyService.save(communityCommentReply);
        } catch (Exception e) {
            System.out.println(e.getMessage());
            throw new ProjectException();
        }


        return AjaxResult.success(communityCommentReply);
    }

    /**
     * 删除回复
     */
    @ApiOperation("删除回复")
    @DeleteMapping("/deleteReply")
    @Transactional
    //@Anonymous
    public AjaxResult deleteReply(String replyId) {
        if (Objects.isNull(replyId)) {
            return AjaxResult.error("参数异常!");
        }
        try {
            CommunityCommentReply communityCommentReply = communityCommentReplyService.getOne(new QueryWrapper<CommunityCommentReply>().eq("id", replyId).and((wrapper) -> {
                wrapper.ne("is_delete", true).or().isNull("is_delete");
            }));
            if (Objects.isNull(communityCommentReply)) {
                return AjaxResult.error("未找到该回复!");
            }
            Long userId = SecurityUtils.getUserId();
            communityCommentReply.setDelete(true);
            communityCommentReply.setUpdateBy(userId);
            communityCommentReply.setUpdateTime(DateUtils.parseDate(DateUtils.getTime()));
            communityCommentReplyService.updateById(communityCommentReply);
        } catch (Exception e) {
            System.out.println(e.getMessage());
            throw new ProjectException();
        }
        return AjaxResult.success("删除回复成功!");
    }


    /**
     * @param articleId 文章id
     * @return 抽奖结果
     */
    @ApiOperation("评论抽奖")
    @GetMapping("/raffleComment")
    @Transactional
    //@Anonymous
    public AjaxResult raffleComment(Long articleId) {
        if (Objects.isNull(articleId)) {
            return AjaxResult.error("参数异常!");
        }

        CommunityArticle communityArticle = communityArticleMapper.selectOne(new QueryWrapper<CommunityArticle>()
                .eq("id", articleId)
                .eq("create_by", SecurityUtils.getUserId())
                .and((wrapper) -> {
                    wrapper.ne("is_delete", true).or().isNull("is_delete");
                }));
        if (communityArticle == null) {
            return AjaxResult.error("非本人无法操作抽奖按钮!");
        }

        CommunityCommentRaffleVo communityCommentRaffleVos = null;
        try {
            communityCommentRaffleVos = communityArticleService.selectCommentRaffle(articleId);

        } catch (Exception e) {
            System.out.println(e.getMessage());
            throw new ProjectException();
        }
        return AjaxResult.success(communityCommentRaffleVos);
    }


    @ApiOperation("查询评论")
    @GetMapping("/queryComment")
    @Transactional
    //@Anonymous
    public AjaxResult queryComment(Long articleId, String comment) {
        int pageNum = Convert.toInt(ServletUtils.getParameter("pageNum"), 1);
        int pageSize = Convert.toInt(ServletUtils.getParameter("pageSize"), 10);
        if (Objects.isNull(articleId)) {
            return AjaxResult.error("参数异常!");
        }
        Long userId = SecurityUtils.getUserId();
        List<CommunitySearchCommentVo> commentVos = new ArrayList<>();
        try {
            int offset = (pageNum - 1) * pageSize;
            //获取一二级评论
            List<CommunityArticleComment> communityArticleComments = communityArticleCommentMapper.queryComment(articleId, comment, offset, pageSize);
            //一二级评论合集Vo
            CommunitySearchCommentVo commentVo = null;
            for (CommunityArticleComment communityArticleComment : communityArticleComments) {
                commentVo = new CommunitySearchCommentVo();
                BeanUtils.copyProperties(communityArticleComment, commentVo);
                commentVos.add(commentVo);
            }
            for (CommunitySearchCommentVo vo : commentVos) {
                //拼接头像 信息
                vo.setAvatar(sysUserService.selectUserById(vo.getUserId()).getAvatar());
                vo.setNickName(sysUserService.selectUserById(vo.getUserId()).getNickName());
                vo.setCommentLikeCount(communityCommentLikeMapper.selectList(new QueryWrapper<CommunityCommentLike>().eq("comment_id", vo.getId())).size());
                //设置当前用户是否点赞
                List<CommunityCommentLike> commentLikes = communityCommentLikeMapper.selectList(new QueryWrapper<CommunityCommentLike>()
                        .eq("comment_id", vo.getId())
                        .eq("user_id", userId));
                vo.setCommentLike(!commentLikes.isEmpty());
                //如果是二级评论则执行这个接口
                CommunityCommentReply communityCommentReply = communityCommentReplyMapper.selectOne(new QueryWrapper<CommunityCommentReply>()
                        .eq("id", vo.getId())
                        .and((wrapper) -> {
                            wrapper.ne("is_delete", true).or().isNull("is_delete");
                        }));

                //如果二级评论不为空执行
                if (!Objects.isNull(communityCommentReply)) {
                    //点赞数量
                    List<CommunityCommentLike> commentLikeList = communityCommentLikeMapper.selectList(new QueryWrapper<CommunityCommentLike>()
                            .eq("reply_id", communityCommentReply.getId()));
                    vo.setCommentLikeCount(commentLikeList.size());

                    //二级评论是否点赞
                    List<CommunityCommentLike> commentLikesReply = communityCommentLikeMapper.selectList(new QueryWrapper<CommunityCommentLike>()
                            .eq("reply_id", vo.getId())
                            .eq("user_id", userId));
                    vo.setCommentLike(!commentLikesReply.isEmpty());

                    //找到一级评论获取 父评论 用户头像和ID
                    CommunityArticleComment communityArticleComment = communityArticleCommentMapper.selectOne(new QueryWrapper<CommunityArticleComment>()
                            .eq("id", communityCommentReply.getCommentId())
                            .and((wrapper) -> {
                                wrapper.ne("is_delete", true).or().isNull("is_delete");
                            }));
                    SysUser sysUser1 = sysUserService.selectUserById(communityArticleComment.getUserId());
                    vo.setParentNickName(sysUser1.getNickName());
                    vo.setParentUserId(sysUser1.getUserId());
                    vo.setParentAvatar(sysUser1.getAvatar());
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
            System.out.println(e.getMessage());
            throw new ProjectException();
        }
        System.out.println(commentVos);
        return AjaxResult.success(commentVos);
    }

    /**
     * 获取当前登录用户的评论和回复
     *
     * @return 评论和回复
     */
    @GetMapping("/getComments")
    @ApiOperation("获取当前登录用户的评论和回复")
    public AjaxResult getCommentInfoByUserId() {
        Long userId = SecurityUtils.getUserId();
        return success(communityArticleCommentService.getCommentInfoByUserId(userId));
    }
}
