package com.ruoyi.generator.controller;

import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.exception.user.ProjectException;
import com.ruoyi.common.utils.SecurityUtils;
import com.ruoyi.generator.domain.Community.CommunityUserNotification;
import com.ruoyi.generator.domain.Community.CommunityUserPrivacy;
import com.ruoyi.generator.service.ICommunityNotificationService;
import com.ruoyi.generator.service.ICommunityPrivacyService;
import com.ruoyi.generator.vo.CommunityUserNotificationVo;
import com.ruoyi.generator.vo.CommunityUserPrivacyVo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.List;
import java.util.Objects;

/**
 * 社区设置管理
 *
 * @author fangQing
 */
@Api(tags = "社区设置管理")
@RestController
@RequestMapping("/community/settings")
public class CommunitySettingsController {


    @Autowired
    private ICommunityNotificationService communityNotificationService;

    @Autowired
    private ICommunityPrivacyService communityPrivacyService;


    /**
     * 获取通知权限
     * @param
     * @return 获取通知权限
     */
    @ApiOperation("获取通知权限")
    @GetMapping("/userNotification")
    //@Anonymous
    public AjaxResult userNotification() {
        List<CommunityUserNotificationVo> communityUserNotificationVos = null;
        try {
            communityUserNotificationVos = communityNotificationService.selectUserNotification(SecurityUtils.getLoginUser().getUserId());
        } catch (Exception e) {
            System.out.println(e.getMessage());
            throw new ProjectException();
        }

        return AjaxResult.success(communityUserNotificationVos);
    }

    /**
     * 更新通知权限
     * @param communityUserNotification
     * @return 返回修改成功的对象
     */
    @ApiOperation("更新通知权限")
    @PostMapping("/modifyUserNotification")
    public AjaxResult modifyUserNotification(@RequestBody CommunityUserNotification communityUserNotification) {
        List<CommunityUserNotificationVo> communityUserNotificationVos = null;
        try {
            communityUserNotificationVos = communityNotificationService.modifyUserNotification(communityUserNotification);
        } catch (Exception e) {
            System.out.println(e.getMessage());
            throw new ProjectException();
        }

        return AjaxResult.success(communityUserNotificationVos);
    }

    /**
     * 获取隐私权限
     * @param
     * @return 获取隐私权限
     */
    @ApiOperation("获取隐私权限")
    @GetMapping("/userPrivacy")
    //@Anonymous
    public AjaxResult userPrivacy(@RequestParam(required = false) Long userId) {
        if (Objects.isNull(userId)) {
            userId = SecurityUtils.getLoginUser().getUserId();
        }

        List<CommunityUserPrivacyVo> communityUserPrivacyVos = null;
        try {
            communityUserPrivacyVos = communityPrivacyService.selectUserPrivacy(userId);
        } catch (Exception e) {
            System.out.println(e.getMessage());
            throw new ProjectException();
        }

        return AjaxResult.success(communityUserPrivacyVos);
    }

    /**
     * 更新隐私权限
     * @param communityUserPrivacy
     * @return 返回修改成功的对象
     */
    @ApiOperation("更新通知权限")
    @PostMapping("/modifyUserPrivacy")
    public AjaxResult modifyUserNotification(@RequestBody CommunityUserPrivacy communityUserPrivacy) {
        List<CommunityUserPrivacyVo> communityUserPrivacyVos = null;
        try {
            communityUserPrivacyVos = communityPrivacyService.modifyUserPrivacy(communityUserPrivacy);
        } catch (Exception e) {
            System.out.println(e.getMessage());
            throw new ProjectException();
        }

        return AjaxResult.success(communityUserPrivacyVos);
    }
}
