package com.ruoyi.generator.service;

import com.alibaba.fastjson2.JSONArray;
import com.alibaba.fastjson2.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.common.utils.SecurityUtils;
import com.ruoyi.generator.domain.Community.*;
import com.ruoyi.generator.mapper.community.*;
import com.ruoyi.generator.vo.CommunityCollectionArticleVo;
import com.ruoyi.generator.vo.CommunityCollectionVo;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;

@Service
public class CommunityCollectionServiceImpl extends ServiceImpl<CommunityCollectionMapper, CommunityCollection> implements ICommunityCollectionService {

    @Autowired
    private CommunityCollectionArticleMapper collectionArticleMapper;

    @Autowired
    private CommunityArticleMapper communityArticleMapper;

    @Autowired
    private CommunityLikeMapper communityLikeMapper;

    @Autowired
    private CommunityArticleCommentMapper communityArticleCommentMapper;

    @Autowired
    private CommunityArticleImagesMapper communityArticleImagesMapper;

    @Autowired
    private CommunityCollectionMapper communityCollectionMapper;

    @Autowired
    private CommunityCollectionArticleMapper communityCollectionArticleMapper;
    @Autowired
    private CommunityCollectionUserMapper communityCollectionUserMapper;

    /**
     * 添加文章到用户合集中
     *
     * @param collectionArticle 合集列表
     */
    @Override
    public void addArticleToCollection(CommunityCollectionArticle collectionArticle) {
        collectionArticleMapper.insert(collectionArticle);
    }

    /**
     * 删除合集中的某个文章
     *
     * @param collectionId 合集id
     * @param articleId    文章id
     */
    @Override
    public void deleteArticleToCollection(Long collectionId, Long articleId) {
        //删除合集中的文章
        CommunityCollectionArticle collectionArticle = new CommunityCollectionArticle();
        collectionArticle.setDelete(true);
        collectionArticle.setUpdateTime(DateUtils.parseDate(DateUtils.getTime()));
        collectionArticle.setUpdateBy(SecurityUtils.getUserId());
        communityCollectionArticleMapper.update(collectionArticle, new UpdateWrapper<CommunityCollectionArticle>()
                .eq("collection_id", collectionId)
                .eq("article_id", articleId).and((wrapper) -> {
                    wrapper.ne("is_delete", true)
                            .or()
                            .isNull("is_delete");
                }));
    }

    /**
     * 根据合集id,查询合集下所有文章信息
     *
     * @param collectionId 合集id
     * @return 文章信息
     */
    @Override
    public JSONArray selectArticleInfoInCollection(Long collectionId, Long searchType) {
        if (searchType == null) {
            searchType = 1L;
        }
        JSONArray jsonArray = new JSONArray();
        List<CommunityCollectionArticleVo> collectionArticleVos = new ArrayList<>();
        //查询合集表
        CommunityCollection communityCollection = communityCollectionMapper.selectOne(new QueryWrapper<CommunityCollection>()
                .eq("id", collectionId)
                .and((wrapper) -> {
                    wrapper.ne("is_delete", true)
                            .or()
                            .isNull("is_delete");
                }));

        if (communityCollection == null) {
            return null;
        }
        //没调用这一个接口 热度 + 1
        UpdateWrapper<CommunityCollection> updateWrapper = new UpdateWrapper<>();
        updateWrapper.eq("id", collectionId)
                .and(w -> w.ne("is_delete", true).or().isNull("is_delete"));
        updateWrapper.setSql("heat = IFNULL(heat, 0) + 1");
        communityCollectionMapper.update(null, updateWrapper);




        //一级层级
        JSONObject collectionObject = new JSONObject();
        collectionObject.put("CollectionName", communityCollection.getCollectionName());
        Date updateTime = communityCollection.getUpdateTime();
        if (updateTime == null) {
            updateTime = communityCollection.getCreateTime();
        }
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        String formattedUpdateTime = sdf.format(updateTime);
        collectionObject.put("collectioncreateby", communityCollection.getCreateBy());
        collectionObject.put("updateTime", formattedUpdateTime);
        collectionObject.put("collectionImageUrl",communityCollection.getImages());
        collectionObject.put("followNumber", communityCollectionUserMapper.selectList(new QueryWrapper<CommunityCollectionUser>().eq("collection_id", collectionId).and((wrapper) -> {
            wrapper.ne("is_delete", true)
                    .or()
                    .isNull("is_delete");
        })).size());
        collectionObject.put("heat", communityCollection.getHeat());
        //浏览量设置默认值
        collectionObject.put("articlePageViews",0);
        collectionObject.put("articleNumber", 0);
        collectionObject.put("articleNumber", 0);


        //查询当前用户是否订阅
        CommunityCollectionUser communityCollectionUser = communityCollectionUserMapper.selectOne(new QueryWrapper<CommunityCollectionUser>()
                .eq("collection_id", collectionId)
                .eq("user_id", SecurityUtils.getUserId())
                .and((wrapper) -> {
                    wrapper.ne("is_delete", true)
                            .or()
                            .isNull("is_delete");
                }));
        boolean follow = false;
        if (communityCollectionUser != null) {
            follow = true;
        }
        collectionObject.put("follow", follow);
        collectionObject.put("Article", collectionArticleVos);
        //查询合集关联文章表
        List<Object> articleIdsObj = collectionArticleMapper
                .selectObjs(new QueryWrapper<CommunityCollectionArticle>()
                        .select("article_id")
                        .eq("collection_id", collectionId)
                        .and((wrapper) -> {
                            wrapper.ne("is_delete", true)
                                    .or()
                                    .isNull("is_delete");
                        }));

        if (articleIdsObj.isEmpty()) {
            return JSONArray.of(collectionObject);
        }

        //转换object为long
        List<Long> articleIds = articleIdsObj.stream()
                .filter(item -> item instanceof Long)
                .map(item -> (Long) item)
                .collect(Collectors.toList());


        List<CommunityArticle> communityArticles;

        if (searchType == 1) {
            // 查询最新的文章
            communityArticles = communityArticleMapper.selectList(
                    new QueryWrapper<CommunityArticle>().in("id", articleIds).and((wrapper) -> {
                        wrapper.ne("is_delete", true)
                                .or()
                                .isNull("is_delete");
                    }).orderByDesc("create_time"));
        } else {
            // 查询最早的文章
            communityArticles = communityArticleMapper.selectList(
                    new QueryWrapper<CommunityArticle>().in("id", articleIds).and((wrapper) -> {
                        wrapper.ne("is_delete", true)
                                .or()
                                .isNull("is_delete");
                    }).orderByAsc("create_time"));
        }
        collectionObject.put("articleNumber", communityArticles.size());

        //获取浏览量总数
        collectionObject.put("articlePageViews",communityArticles.stream()
                .mapToLong(CommunityArticle::getPageViews)
                .sum());


        CommunityCollectionArticleVo collectionArticleVo = null;
        for (CommunityArticle communityArticle : communityArticles) {
            collectionArticleVo = new CommunityCollectionArticleVo();
            collectionArticleVo.setArticleId(communityArticle.getId());
            collectionArticleVo.setCollectionId(collectionId);
            collectionArticleVo.setTitle(communityArticle.getTitle());
            collectionArticleVo.setArticleCreateTime(communityArticle.getCreateTime());
            collectionArticleVo.setContent(communityArticle.getContent());

            collectionArticleVo.setPageViews(communityArticle.getPageViews());
            //设置用户点赞数
            int likeCount = communityLikeMapper.selectCount(new QueryWrapper<CommunityLike>().eq("article_id", communityArticle.getId())).intValue();
            collectionArticleVo.setLikeCount(likeCount);
            //设置用户评论数
            int commentCount = communityArticleCommentMapper.selectCount(new QueryWrapper<CommunityArticleComment>().eq("article_id", communityArticle.getId()).and((wrapper) -> {
                wrapper.ne("is_delete", true)
                        .or()
                        .isNull("is_delete");
            })).intValue();
            collectionArticleVo.setCommentCount(commentCount);
            //设置文章封面
            List<CommunityArticleImages> articleImages = communityArticleImagesMapper.selectList(new QueryWrapper<CommunityArticleImages>()
                    .select("image_url")
                    .eq("article_id", communityArticle.getId()));
            if (!articleImages.isEmpty()) {
                collectionArticleVo.setImageUrl(articleImages.get(0).getImageUrl());
            }

            collectionArticleVos.add(collectionArticleVo);
        }


        collectionObject.put("Article", collectionArticleVos);
        jsonArray.add(collectionObject);

        return jsonArray;
    }

    /**
     * 删除合集
     *
     * @param collectionId 合集id
     */
    @Override
    public void deleteCollectionById(Long collectionId) {
        //1.更新合集状态
        CommunityCollection collection = communityCollectionMapper.selectOne(new QueryWrapper<CommunityCollection>()
                .eq("id", collectionId)
                .and((wrapper) -> {
                    wrapper.ne("is_delete", true)
                            .or()
                            .isNull("is_delete");
                }));
        collection.setDelete(true);
        collection.setUpdateBy(SecurityUtils.getLoginUser().getUserId());
        collection.setUpdateTime(DateUtils.parseDate(DateUtils.getTime()));
        communityCollectionMapper.updateById(collection);
        //2.更新合集关联的文章状态
        List<Long> ids = communityCollectionArticleMapper.selectList(new QueryWrapper<CommunityCollectionArticle>()
                .eq("collection_id", collectionId)
                .and((wrapper) -> {
                    wrapper.ne("is_delete", true)
                            .or()
                            .isNull("is_delete");
                })).stream().map(item -> item.getId()).collect(Collectors.toList());
        if (!ids.isEmpty()) {
            CommunityCollectionArticle collectionArticle = new CommunityCollectionArticle();
            collectionArticle.setDelete(true);
            collectionArticle.setUpdateBy(SecurityUtils.getLoginUser().getUserId());
            collectionArticle.setUpdateTime(DateUtils.parseDate(DateUtils.getTime()));
            communityCollectionArticleMapper.update(collectionArticle, new UpdateWrapper<CommunityCollectionArticle>().in("id", ids));
        }
    }

    @Override
    public List<CommunityCollectionVo> selectUserCollection(Long userId, Long articleId, int pageNum, int pageSize, int searchType) {
        int offset = (pageNum - 1) * pageSize;
        List<CommunityCollectionVo> communityCollectionVos = communityCollectionMapper.selectUserCollection(userId, articleId, offset, pageSize, searchType);
        for (CommunityCollectionVo communityCollectionVo : communityCollectionVos) {
            Long aLong = communityCollectionArticleMapper.selectCount(new QueryWrapper<CommunityCollectionArticle>()
                    .eq("collection_id", communityCollectionVo.getId())
                    .and((wrapper) -> {
                        wrapper.ne("is_delete", true)
                                .or()
                                .isNull("is_delete");
                    }));
            communityCollectionVo.setArticleCount(aLong);
        }
        return communityCollectionVos;
    }


}
